#ifndef __CODA_GMTIME_H__
#define __CODA_GMTIME_H__

#include <stdint.h>
#include <time.h>

static inline
int coda_msleep(int ms)
{
	struct timespec ts;

	ts.tv_sec  = (ms / 1000);
	ts.tv_nsec = (ms % 1000) * 1000000;

	return nanosleep(&ts, NULL);
}

static inline
void coda_gmtime(time_t t, struct tm *tp)
{
	int yday;
	uintptr_t n, sec, min, hour, mday, mon, year, wday, days, leap;

	/* the calculation is valid for positive time_t only */

	n = (uintptr_t) t;
	days = n / 86400;

	/* Jaunary 1, 1970 was Thursday */

	wday = (4 + days) % 7;
	n %= 86400;
	hour = n / 3600;
	n %= 3600;
	min = n / 60;
	sec = n % 60;

	/*
	 * the algorithm based on Gauss' formula,
	 * see src/http/ngx_http_parse_time.c
	 */

	/* days since March 1, 1 BC */
	days = days - (31 + 28) + 719527;

	/*
	 * The "days" should be adjusted to 1 only, however, some March 1st's go
	 * to previous year, so we adjust them to 2.  This causes also shift of the
	 * last Feburary days to next year, but we catch the case when "yday"
	 * becomes negative.
	 */

	year = (days + 2) * 400 / (365 * 400 + 100 - 4 + 1);
	yday = days - (365 * year + year / 4 - year / 100 + year / 400);

	if (yday < 0)
	{
		leap = (year % 4 == 0) && (year % 100 || (year % 400 == 0));
		yday = 365 + leap + yday;
		year--;
	}

	/*
	 * The empirical formula that maps "yday" to month.
	 * There are at least 10 variants, some of them are:
	 *     mon = (yday + 31) * 15 / 459
	 *     mon = (yday + 31) * 17 / 520
	 *     mon = (yday + 31) * 20 / 612
	 */

	mon = (yday + 31) * 10 / 306;

	/* the Gauss' formula that evaluates days before the month */

	mday = yday - (367 * mon / 12 - 30) + 1;

	if (yday >= 306)
	{
		year++;
		mon -= 10;

		/*
		 * there is no "yday" in Win32 SYSTEMTIME
		 *
		 * yday -= 306;
		 */
	}
	else
	{
		mon += 2;

		/*
		 * there is no "yday" in Win32 SYSTEMTIME
		 *
		 * yday += 31 + 28 + leap;
		 */
	}

	tp->tm_sec  = (int) sec;
	tp->tm_min  = (int) min;
	tp->tm_hour = (int) hour;
	tp->tm_mday = (int) mday;
	tp->tm_mon  = (int) mon;
	tp->tm_year = (int) year;
	tp->tm_wday = (int) wday;
}

#endif /* __CODA_GMTIME_H__ */
